import java.util.Vector;
import java.io.File;
import javax.swing.filechooser.FileFilter;

/**
 * This file filter matches all files with a given set of extensions.
 */

/**
 * AUTHOR: MUHAMMAD ADNAN
 * LICENSE: GNU General Public License, version 3 (GPL-3.0)
*/

class ExtensionFileFilter extends FileFilter
{
   private String description = "";
   private Vector<String> extensions = new Vector<String>();

   /**
     * Adds an extension that this file filter recognizes.
     * @param extension a file extension (such as ".txt" or "txt")
     */
   public void addExtension(String extension)
   {
      if (!extension.startsWith("."))
         extension = "." + extension;
      extensions.add(extension.toLowerCase());
   }

   /**
     * Sets a description for the file set that this file filter
     * recognizes.
     * @param aDescription a description for the file set
     */
   public void setDescription(String aDescription)
   {
      description = aDescription;
   }

   /**
      Returns a description for the file set that this file
      filter recognizes.
      @return a description for the file set
   */
   public String getDescription()
   {
      return description;
   }

   public boolean accept(File f)
   {
      if (f.isDirectory()) return true;
      String name = f.getName().toLowerCase();

      // check if the file name ends with the extension given
      for (int i = 0; i < extensions.size(); i++)
      {
         if (name.endsWith((String)extensions.get(i)))
         {
            return true;
         }
      }
      return false;
   }
}
