import java.awt.GridBagLayout;
import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.BorderFactory;
import java.io.IOException;
import javax.swing.JFileChooser;


/**
 * A panel for the user to specify the postcode column in the data file.
 * The postcode column will be used to match records with the ONSPD.
 */

/**
 * AUTHOR: MUHAMMAD ADNAN
 * LICENSE: GNU General Public License, version 3 (GPL-3.0)
*/

public class IdentificationPanel extends JPanel implements ActionListener
{

	private static final long serialVersionUID = 10027;

	private ReadFile readFiles;

	public static int indexVal=0;

	private int[] indexes;

	private JFileChooser chooser;

    private int[] columnNr;

    /** the frame of the program */
    private Screen parent;

    private ReadFile rf;

    /** the name of the input file */
    private String fileName;

    /** the headers of the column file */
    private String[] fileHeaders;

    /** the first fifty records of the file */
    private String[][] fiftyRecords;

    /** the drop down menus containing the headers of the input file */
    private JComboBox[] comboBoxes;

    /** checkboxes for each of the possible import fields */
    private JCheckBox[] checkBoxes;

    private static int nrProcesses = 0;

    private String filePath;

    private static final int NR_IMPORT_FIELDS = 3;

    private static final int NR_MANDATORY_FIELDS = 2;

    public IdentificationPanel(Screen parent, ReadFile rf, String fileName,String[][] fiftyRecords, String[] fileHeaders)
      {


		for(int i=0;i<fileHeaders.length;i++)
		{
			String a=fileHeaders[i];
			String b=a;

			if(a.length()<50)
			 {
				 for(int j=a.length();j<=50;j++)
				  {
					  b+=" ";
				  }

			 }
			fileHeaders[i]=b;
		}

        setOpaque(true);
        setBackground(Color.WHITE);
        setLayout(new GridBagLayout());

        this.parent = parent;
        this.rf = rf;
        this.fileName = fileName;
        this.fileHeaders = fileHeaders;
        this.fiftyRecords = fiftyRecords;

        parent.setSize(500,420);

        comboBoxes = new JComboBox[NR_IMPORT_FIELDS];
        checkBoxes = new JCheckBox[NR_IMPORT_FIELDS];

        ActionListener compute = new ComputeListener();
        ActionListener back = new BackListener();

        columnNr = getIndexes(fileHeaders);


        for(int i = 0; i < NR_IMPORT_FIELDS; i++)
        {
            comboBoxes[i] = new JComboBox(fileHeaders);
            checkBoxes[i] = new JCheckBox();
            checkBoxes[i].addActionListener(this);
            checkBoxes[i].setOpaque(true);
            checkBoxes[i].setBackground(Color.WHITE);
        }

        JLabel label[] = new JLabel[NR_IMPORT_FIELDS];
        label[0] = new JLabel("Postcode:");
        label[1] = new JLabel("Surname:");
        label[2] = new JLabel("Forename:");

        for (int i = 0; i < 1; i++)
        {
            if (columnNr[i] > -1)
            {
                checkBoxes[i].setSelected(true);
                comboBoxes[i].setSelectedItem(fileHeaders[columnNr[i]]);
            }
            else
            {
                checkBoxes[i].setSelected(false);
                comboBoxes[i].setSelectedItem(null);
            }
        }

        /** select the mandatory fields */
        for(int i = 0; i < 1; i++) {
            checkBoxes[i].setSelected(true);
        }

        JButton computeButton = new JButton(" Next >>");
        computeButton.addActionListener(compute);

        JButton backButton = new JButton("<< Back ");
        backButton.addActionListener(back);

        JLabel menu = new JLabel("Select Postcode");
        menu.setFont(new Font("SansSerif", Font.BOLD, 20));

        // add components to grid, using GBC convenience class
        add(menu, new GBC(0, 0, 3, 1).setInsets(0, 20, 30, 20));
        add(new JLabel("Please select the postcode column in your csv file."),
                new GBC(0, 1, 3, 1).setAnchor(GridBagConstraints.WEST).setFill(GBC.HORIZONTAL).setInsets(0, 0, 5, 60));
        add(new JLabel(""),
                new GBC(0, 2, 3, 1).setAnchor(GridBagConstraints.WEST).setFill(GBC.HORIZONTAL).setInsets(0, 0, 5, 60));
        add(new JLabel(""),
                new GBC(0, 3, 3, 1).setAnchor(GridBagConstraints.WEST).setFill(GBC.HORIZONTAL).setInsets(0, 0, 5, 60));
        add(new JLabel(""),
                new GBC(0, 4, 3, 1).setAnchor(GridBagConstraints.WEST).setFill(GBC.HORIZONTAL).setInsets(0, 0, 30, 60));
        add(new JLabel(""), new GBC(2, 4).setInsets(10));

        Font labelFont = new Font("SansSerif", Font.BOLD, 12);

        JLabel imp = new JLabel("");
        imp.setFont(labelFont);
        JLabel fn = new JLabel("");
        fn.setFont(labelFont);
        JLabel fcn = new JLabel("");
        fcn.setFont(labelFont);

        /////////////////
        chooser = new JFileChooser("");
        final ExtensionFileFilter filter = new ExtensionFileFilter();
		filter.addExtension("csv");
        filter.setDescription("Comma Separated Value files");

        chooser.setFileFilter(filter);
		chooser.setDialogTitle("Specify the export file name and location.");
        /////////////////

        for (int i = 0; i < 1; i++)
        {
            add(comboBoxes[i], new GBC(0, 6 + i).setInsets(0, 0, 20, 30));
        }

        JPanel hyperlinksPanel = new JPanel();
        hyperlinksPanel.setOpaque(true);
        hyperlinksPanel.setBackground(Color.WHITE);

        add(hyperlinksPanel,new GBC(0, 10, 3, 1).setAnchor(GridBagConstraints.WEST).setInsets(20, 0, 30, 60));
        add(backButton, new GBC(1, 13).setAnchor(GridBagConstraints.WEST).setInsets(0, 20, 30, 10));
        add(computeButton, new GBC(2, 13).setAnchor(GridBagConstraints.WEST).setInsets(0, 10, 30, 20));
    }

    public void actionPerformed(ActionEvent event)
    {
        JCheckBox e = (JCheckBox) event.getSource();

        for(int i = 0; i < NR_MANDATORY_FIELDS; i++)
        {
            if (e == checkBoxes[i])
            {
                e.setSelected(true);
                JOptionPane.showMessageDialog(new ErrorFrame(),"This field is mandatory.", "Warning message",JOptionPane.WARNING_MESSAGE);
            }
        }
    }


    private class BackListener implements ActionListener
    {

        public void actionPerformed(ActionEvent event)
        {
            removeAll();
            validate();
            parent.getContentPane().removeAll();
            parent.add(BorderLayout.NORTH,new LogoPanel("images/logo.bmp"));
            parent.getContentPane().add(new ProcessTablePanel(parent, rf, fileName, fiftyRecords, fileHeaders));
            parent.centerFrame();
            parent.validate();
        }
    }


    private class ComputeListener implements ActionListener
    {

        public void actionPerformed(ActionEvent event)
        {
            goToNextPanel();
        }
    }

    private class NextMouseListener extends MouseAdapter
    {

        public void mouseClicked(MouseEvent event)
        {
            goToNextPanel();
        }
    }

    public void goToNextPanel()
    {
        if (comboBoxes[0].getSelectedIndex() < 0)
        {
            JOptionPane.showMessageDialog(new ErrorFrame(), "Please, " +  "specify the column names of at least the Postcode " + "field.", "Error Message",
                    JOptionPane.ERROR_MESSAGE);
        }
        else
        {

		     removeAll();
			 validate();
			 parent.getContentPane().removeAll();
			 parent.add(BorderLayout.NORTH,new LogoPanel("images/logo.bmp"));

			 boolean writeScore = false;
			 boolean writeCases = false;
             indexVal=comboBoxes[0].getSelectedIndex();
             parent.setSize(500,420);

             parent.getContentPane().add(new FinalPanel(parent,readFiles, fileName, fiftyRecords, fileHeaders));
             parent.validate();

        }
    }

    public int[] getIndexes(String[] inputHeaders)
    {
        int[] indexes = new int[NR_IMPORT_FIELDS];

        for (int i = 0; i < indexes.length; i++)
        {
            indexes[i] = -1;
        }

        for (int i = inputHeaders.length - 1; i >= 0; i--) {
            if ((inputHeaders[i].toLowerCase().indexOf("id") > -1)
                    || (inputHeaders[i].toLowerCase().indexOf("nr") > -1)
                    || (inputHeaders[i].toLowerCase().indexOf("#") > -1)
                    || (inputHeaders[i].toLowerCase().indexOf("post") > -1)
                    || (inputHeaders[i].toLowerCase().indexOf("Post") > -1)) {
                indexes[0] = i;
            } else if ((inputHeaders[i].toLowerCase().indexOf("sur") > -1)
                    || (inputHeaders[i].toLowerCase().indexOf("last") > -1)) {
                indexes[1] = i;
            } else if ((inputHeaders[i].toLowerCase().indexOf("fore") > -1)
                    || (inputHeaders[i].toLowerCase().indexOf("christian") > -1)
                    || (inputHeaders[i].toLowerCase().indexOf("first") > -1)) {
                indexes[2] = i;
            }

        }

        return indexes;
    }
}
