import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.ImageIcon;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.BorderFactory;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import java.awt.GridBagConstraints;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import java.io.File;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import javax.swing.*;

/**
 * This panel is shown when the user starts the OACoder.
 * This panel i used for the user to specify the input data file.
*/

/**
 * AUTHOR: MUHAMMAD ADNAN
 * LICENSE: GNU General Public License, version 3 (GPL-3.0)
*/

public class LoadFilePanel extends JPanel
{

	private static final long serialVersionUID = 10031;

    /** The frame in which this panel is displayed */
    private Screen parent;


    private ReadFile file;
    private JFileChooser chooser;

    /** displays the panel with the processes */
    private JButton loadButton;

    /** displays the settings panel */
    private JButton settingsButton;


    /** describes the function of the statistics button */
    private JLabel statisticsLabel;

    //public static int nrLoadPanels = 0;


    private JTextField fileField;

    public static String defaultWorkplace = null;

    /**
     * contains the columns that will be exported from the input file
     * into the export file. For example, if the second column is selected
     * (0, 1, 2, 3, 5, 6), then 1 is stored.
     */
    private int[] columnNr;

    private boolean computationFinished;

    /**  */
    public static int whichLoadInfo = 0;

    /**  */
    private static int nrLoadPanels = 0;

    /**
     *
     */
    public LoadFilePanel(Screen parent)
    {

        this.parent = parent;
        whichLoadInfo = 0;
        initialize(LoadFilePanel.this);
        parent.setSize(510,430);

    }

    public LoadFilePanel(Screen parent, ReadFile rf, int[] columnNr, boolean computationFinished)
    {
        this.parent = parent;
        this.columnNr = columnNr;
        this.computationFinished = computationFinished;
        file = rf;
        whichLoadInfo = 2;
        initialize(LoadFilePanel.this);



        if (computationFinished)
        {
            statisticsLabel.setEnabled(true);
        }
    }

    public LoadFilePanel(Screen parent, ReadFile rf)
    {
        whichLoadInfo = 1;
        this.parent = parent;
        file = rf;
        initialize(LoadFilePanel.this);
    }

    public JPanel initialize(JPanel pane)
    {
        nrLoadPanels++;

        setOpaque(true);
        setBackground(Color.WHITE);

        ActionListener loadListener = new LoadListener();
        loadButton = new JButton("Read file");
        loadButton.addActionListener(loadListener);

        ActionListener settingsListener = new SettingsListener();
        settingsButton = new JButton("Browse");
        settingsButton.addActionListener(settingsListener);

        JLabel menu = new JLabel("Public Profiler - OACoder");
        menu.setFont(new Font("SansSerif", Font.BOLD, 20));


       if(defaultWorkplace != null)
       {
		chooser = new JFileChooser(defaultWorkplace);
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		chooser.addChoosableFileFilter(new TextFileFilter());
	   }
	   else
	   {
		chooser = new JFileChooser();
		chooser.changeToParentDirectory();
		defaultWorkplace = chooser.getCurrentDirectory().getParent();
		chooser.addChoosableFileFilter(new TextFileFilter());
       }

        fileField=new JTextField(30);


        JLabel helpInfo = new JLabel(" OAC coder enables you to join Output Area Classification (OAC) codes to a csv file of postcodes.");

        JPanel sentenceWithLink = new JPanel();
        sentenceWithLink.setOpaque(true);
        sentenceWithLink.setBackground(Color.WHITE);

        JLabel main = new JLabel("        OACoder");
        main.setFont(new Font("SansSerif", Font.BOLD, 25));


        JLabel info = new JLabel("Browse for File:");

        setLayout(null);
        pane.add(main);

        main.setBounds(120,10,400,50);

        helpInfo.setBounds(10,100,500,20);
        pane.add(helpInfo);

        info.setBounds(30,140,100,20);
        pane.add(info);

        fileField.setBounds(140,140,200,20);
        pane.add(fileField);

        settingsButton.setBounds(350,140,100,20);
        pane.add(settingsButton);

        loadButton.setBounds(350,170,100,20);
        pane.add(loadButton);

        JLabel main1 = new JLabel("* Contains Ordnance Survey data  Crown copyright and database right 2010");
        main1.setFont(new Font("SansSerif", Font.BOLD, 9));

        main1.setBounds(122,250,370,20);
        pane.add(main1);

        JLabel main5 = new JLabel("* Contains Royal Mail data  Royal Mail copyright and database right 2010");
		main5.setFont(new Font("SansSerif", Font.BOLD, 9));

		main5.setBounds(122,260,370,20);
        pane.add(main5);

        JLabel main6 = new JLabel("* Contains National Statistics data  Crown copyright and database right 2010");
		main6.setFont(new Font("SansSerif", Font.BOLD, 9));

		main6.setBounds(122,270,370,20);
        pane.add(main6);

        pane.setVisible(true);

        return pane;
    }

    public static int getNrLoadFilePanels()
    {
        return nrLoadPanels;
    }

    public void setFile(ReadFile rf)
    {
        file = rf;
    }


    private class SettingsListener implements ActionListener
    {

        public void actionPerformed(ActionEvent evt)

        {

            int result  = chooser.showOpenDialog(parent);

			 // if csv file accepted, set it as icon of the label
			if (result == JFileChooser.APPROVE_OPTION)
			 {

			   File directoryFile = new File("csvFiles/directoryFile.txt");
			   WriteFile wf = null;
			   String directory = null;
			   wf = new WriteFile("csvFiles/directoryFile.txt");
			   directory = chooser.getSelectedFile().getAbsolutePath();
			   defaultWorkplace = directory;

			   if(!directory.substring(directory.length()-3,directory.length()).equals("csv"))
			    {
			   	   JOptionPane.showMessageDialog(new ErrorFrame(), "OACoder only accepts csv files.", "Error Message",JOptionPane.ERROR_MESSAGE);
			    }
			   else
			    {
			       fileField.setText(directory);
			    }

			   try
			   {
			    wf.write(directory);
			    wf.close();
			   }
			   catch (IOException e)
			   {}

            }
        }
    }


    private class LoadListener implements ActionListener
    {

        public void actionPerformed(ActionEvent evt)

        {


			 if((fileField.getText()).equals(""))
		     {
				 JOptionPane.showMessageDialog(new ErrorFrame(), "File is not selected.", "Error Message", JOptionPane.ERROR_MESSAGE);

			 }
			 else
			 {
			  removeAll();
			  validate();
			  parent.getContentPane().removeAll();
			  parent.add(BorderLayout.NORTH,new LogoPanel("images/logo.bmp"));
			  parent.getContentPane().add(new FileWaitPanel(parent, file,fileField.getText(), nrLoadPanels));
              parent.validate();
		     }

        }
    }


    private class ExitListener implements ActionListener
    {

        public void actionPerformed(ActionEvent evt)
        {
            System.exit(0);
        }
    }

    public class TextFileFilter extends javax.swing.filechooser.FileFilter
	{
	    public boolean accept(File file)
	     {
	        return (file.getName().toLowerCase().endsWith(".csv")  || file.isDirectory());
	     }

	    public String getDescription()
	     {
	        return "CSV File (*.csv)";
	     }
    }
}


