
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JProgressBar;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.BorderFactory;
import javax.swing.JScrollPane;
import javax.swing.Timer;
import java.awt.GridBagLayout;
import java.awt.Font;
import java.awt.Insets;
import java.awt.GridBagConstraints;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Color;
import java.io.IOException;
import javax.swing.JOptionPane;


/**
 * This panel is the final panel in the 'Postocde' -> 'OAC Code' assignment process
 * This panel shows a progress bar of the status of coding process
 * This panel also calls helper files to code the Postcodes with their corresponding OAC Coders.
*/

/**
 * AUTHOR: MUHAMMAD ADNAN
 * LICENSE: GNU General Public License, version 3 (GPL-3.0)
*/


public class ProgressBarPanel extends JPanel
{

	private static final long serialVersionUID = 10028;

    /**  */
    private static final int DELAY = 5;

    /**  */
    private Screen parent;

    /**  */
    private ReadFile rf;

    /**  */
    private javax.swing.Timer activityMonitor;

    /**  */
    private JProgressBar progressBar;

    /**  */
    private JTextArea taskOutput;

    /**  */
    private JLabel progressLabel;

    /**  */
    private JLabel progressLabel2;

    /**  */
    private JButton cancelButton;

    /**  */
    private JButton backButton;

    /**  */
    private JButton statButton;

    /**  */
    private FileWritePostcodes writeCels;

    /**  */
    private long lastCurrent;

    /**  */
    private int[] columnNr;

    /**  */
    private JButton computeButton;

    /**  */
    private boolean computeIsFinished;

    /**  */
    private int nrActivities;

    /**  */
    private String fileName;

    /**  */
    private String[][] fiftyRecords;

    /**  */
    private String[] fileHeaders;

    /**  */
    private int[] indexes;

    /**  */
    private Thread computation;

    /**  */
    public static final int SECONDS_PER_MINUTE = 60;

    public String fName="";


    public ProgressBarPanel(Screen parent, ReadFile rf, String fileName,String[][] fiftyRecords, String[] fileHeaders, String filePath,int[] columnNr, int[] indexes, int[] exportedCels,boolean writeScore, boolean writeCases)
    {

        GridBagLayout layout = new GridBagLayout();
        setOpaque(true);
        setBackground(Color.WHITE);

        this.parent = parent;
        this.rf = rf;
        this.columnNr = columnNr;
        this.fileName = fileName;
        this.fiftyRecords = fiftyRecords;
        this.fileHeaders = fileHeaders;
        this.indexes = indexes;
        computeIsFinished = true;

        parent.setSize(Screen.WIDTH, Screen.HEIGHT);


        progressBar = new JProgressBar();
        progressBar.setStringPainted(true);

        cancelButton = new JButton(" Cancel ");
        cancelButton.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent event) {
                taskOutput.append("Coding process canceled by the user.\n");
                taskOutput.append("Please click '<<Back' to change your " +
                        "specifications,\n");
                taskOutput.append("or click 'Main Menu' to start again.\n");
                activityMonitor.stop();
                writeCels.setThreadStop(true);
                computeButton.setEnabled(true);
                statButton.setEnabled(true);
                backButton.setEnabled(true);
                cancelButton.setEnabled(false);
                computeIsFinished = false;
            }
        });

        statButton = new JButton("Statistics");
        statButton.addActionListener(new StatisticsListener());
        statButton.setEnabled(false);

        backButton = new JButton("Close  ");
        backButton.addActionListener(new BackListener());
        backButton.setEnabled(false);

        computeButton = new JButton("Show CSV");
        computeButton.addActionListener(new MenuListener());
        computeButton.setEnabled(false);

        progressLabel = new JLabel("Postcodes are being coded to their OAC groups now.");
        progressLabel2 = new JLabel("Please be patient. This may take a few " + "seconds.");

        taskOutput = new JTextArea(10, 30);
        taskOutput.setMargin(new Insets(5,5,5,5));
        taskOutput.setEditable(false);

        progressBar.setMaximum(FileWaitPanel.totalRecords-1);
        progressBar.setIndeterminate(false);
        nrActivities = 0;
        fName=filePath;

	    try
	     {
			 writeCels = new FileWritePostcodes(filePath, fileName);
	     }
	     catch (IOException e)
	     {
			 System.out.println("Error occured: " + e);
		 }

        // set up the timer action
        activityMonitor = new Timer(DELAY, new ActionListener()
        {

            public void actionPerformed(ActionEvent event)
            {
                nrActivities++;
                int current = writeCels.getLineNr();

                //int current=10;

                //show progress
                taskOutput.append((current - 1) + " records coded" + "\n");
                progressBar.setStringPainted(!progressBar.isIndeterminate());
                progressBar.setBackground(Color.BLUE);
                progressBar.setValue(current);

                // check if task is completed
                if (current >= FileWaitPanel.totalRecords-1)
                {
                    taskOutput.append("Postcode-to-OAC coding and file exporting completed!\n");
                    taskOutput.append("Done in " + secondsToHoursMinutesSeconds(nrActivities * DELAY / 1000) + "\n");
                    activityMonitor.stop();

                    if (current == lastCurrent)
                    {
                        progressBar.setStringPainted(progressBar.isIndeterminate());
                    }

                    computeButton.setEnabled(true);
                    statButton.setEnabled(true);
                    backButton.setEnabled(true);
                    cancelButton.setEnabled(false);
                }

                lastCurrent = current;
            }
         });

         computation = new Thread(writeCels);
         computation.start();
         activityMonitor.start();

         JLabel main = new JLabel("Coding postcodes and exporting data");
         main.setFont(new Font("SansSerif", Font.BOLD, 20));


         setLayout(null);

         main.setBounds(100,10,400,50);
		 add(main);

		 progressLabel.setBounds(100,70,400,20);
		 add(progressLabel);

		 progressLabel2.setBounds(100,95,400,20);
		 add(progressLabel2);

         JScrollPane jsp=new JScrollPane(taskOutput);
		 jsp.setBounds(100,130,400,200);
		 add(jsp);

		 progressBar.setBounds(100,340,400,20);
		 add(progressBar);

		 backButton.setBounds(180,400,100,30);
		 add(backButton);

		 computeButton.setBounds(330,400,100,30);
		 add(computeButton);


    }

    public static String secondsToHoursMinutesSeconds(int totalSeconds)
    {
        int seconds = totalSeconds % SECONDS_PER_MINUTE;
        int minutes = ((totalSeconds - seconds) / SECONDS_PER_MINUTE) % SECONDS_PER_MINUTE;
        int hours = totalSeconds / (SECONDS_PER_MINUTE * SECONDS_PER_MINUTE);

        String time = "";

        if (hours < 10)
        {
            time = time + "0" + hours + "h:";
        }
        else
        {
            time = time + hours + "h:";
        }

        if (minutes < 10)
        {
            time = time + "0" + minutes + "m:";
        }
        else
        {
            time = time + minutes + "m:";
        }

        if (seconds < 10)
        {
            time = time + "0" + seconds + "s";
        }
        else
        {
            time = time + seconds + "s";
        }

        return time;
    }


    private class MenuListener implements ActionListener
    {

        public void actionPerformed(ActionEvent event)
        {
            BrowserControl.displayURL(fName);
        }
    }


    private class StatisticsListener implements ActionListener
    {

        public void actionPerformed(ActionEvent event)
        {
            removeAll();
            validate();
            parent.getContentPane().removeAll();
            parent.add("North", new LogoPanel("images/logo.bmp"));
            //parent.getContentPane().add(new MainChartPanel(parent, rf,writeCels, columnNr));
            parent.setSize(Screen.BIG_SCREEN_WIDTH, Screen.BIG_SCREEN_HEIGHT);
            parent.centerFrame();
            parent.validate();
        }
    }

    private class BackListener implements ActionListener
    {

        public void actionPerformed(ActionEvent event)
        {

            removeAll();
            System.exit(0);

        }
    }



}
